@@ -1,44 +1,59 @@
 """
 Utilities for creating bokeh Server instances.
 """
-from __future__ import absolute_import, division, unicode_literals
-
 import datetime as dt
+import html
+import inspect
 import os
+import pathlib
 import signal
 import sys
+import traceback
 import threading
 import uuid
 
 from collections import OrderedDict
 from contextlib import contextmanager
-from functools import partial
+from functools import partial, wraps
 from types import FunctionType, MethodType
+from urllib.parse import urljoin, urlparse
 
 import param
 import bokeh
 import bokeh.command.util
 
+# Bokeh imports
 from bokeh.application import Application as BkApplication
+from bokeh.application.handlers.code import CodeHandler
 from bokeh.application.handlers.function import FunctionHandler
+from bokeh.command.util import build_single_handler_application
+from bokeh.core.templates import AUTOLOAD_JS
 from bokeh.document.events import ModelChangedEvent
-from bokeh.embed.bundle import extension_dirs
+from bokeh.embed.bundle import Script
+from bokeh.embed.elements import html_page_for_render_items, script_for_render_items
+from bokeh.embed.util import RenderItem
 from bokeh.io import curdoc
 from bokeh.server.server import Server
+from bokeh.server.urls import per_app_patterns
+from bokeh.server.views.autoload_js_handler import AutoloadJsHandler as BkAutoloadJsHandler
+from bokeh.server.views.doc_handler import DocHandler as BkDocHandler
+
+# Tornado imports
 from tornado.ioloop import IOLoop
 from tornado.websocket import WebSocketHandler
-from tornado.web import RequestHandler, StaticFileHandler
+from tornado.web import RequestHandler, StaticFileHandler, authenticated
 from tornado.wsgi import WSGIContainer
 
+# Internal imports
+from ..util import edit_readonly
+from .reload import autoreload_watcher
+from .resources import BASE_TEMPLATE, Resources, bundle_resources
 from .state import state
 
 #---------------------------------------------------------------------
 # Private API
 #---------------------------------------------------------------------
 
-# Handle serving of the panel extension before session is loaded
-extension_dirs['panel'] = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'dist'))
-
 INDEX_HTML = os.path.join(os.path.dirname(__file__), '..', '_templates', "index.html")
 
 def _origin_url(url):
@@ -46,38 +61,12 @@ def _origin_url(url):
         url = url.split("//")[1]
     return url
 
-
 def _server_url(url, port):
     if url.startswith("http"):
         return '%s:%d%s' % (url.rsplit(':', 1)[0], port, "/")
     else:
         return 'http://%s:%d%s' % (url.split(':')[0], port, "/")
 
-
-def init_doc(doc):
-    doc = doc or curdoc()
-    if not doc.session_context:
-        return doc
-
-    session_id = doc.session_context.id
-    sessions = state.session_info['sessions']
-    if session_id not in sessions:
-        return doc
-
-    sessions[session_id].update({
-        'started': dt.datetime.now().timestamp()
-    })
-    doc.on_event('document_ready', state._init_session)
-    return doc
-
-
-@contextmanager
-def set_curdoc(doc):
-    state.curdoc = doc
-    yield
-    state.curdoc = None
-
-
 def _eval_panel(panel, server_id, title, location, doc):
     from ..template import BaseTemplate
     from ..pane import panel as as_panel
@@ -91,16 +80,34 @@ def _eval_panel(panel, server_id, title, location, doc):
             doc = as_panel(panel)._modify_doc(server_id, title, doc, location)
         return doc
 
+def async_execute(func):
+    """
+    Wrap async event loop scheduling to ensure that with_lock flag
+    is propagated from function to partial wrapping it.
+    """
+    if not state.curdoc or not state.curdoc.session_context:
+        ioloop = IOLoop.current()
+        event_loop = ioloop.asyncio_loop
+        if event_loop.is_running():
+            ioloop.add_callback(func)
+        else:
+            event_loop.run_until_complete(func())
+        return
 
-class Application(BkApplication):
-
-    async def on_session_created(self, session_context):
-        for cb in state._on_session_created:
-            cb(session_context)
-        await super().on_session_created(session_context)
-
-bokeh.command.util.Application = Application
+    if isinstance(func, partial) and hasattr(func.func, 'lock'):
+        unlock = not func.func.lock
+    else:
+        unlock = not getattr(func, 'lock', False)
+    if unlock:
+        @wraps(func)
+        async def wrapper(*args, **kw):
+            return await func(*args, **kw)
+        wrapper.nolock = True
+    else:
+        wrapper = func
+    state.curdoc.add_next_tick_callback(wrapper)
 
+param.parameterized.async_executor = async_execute
 
 def _initialize_session_info(session_context):
     from ..config import config
@@ -124,10 +131,230 @@ def _initialize_session_info(session_context):
 
 state.on_session_created(_initialize_session_info)
 
+#---------------------------------------------------------------------
+# Bokeh patches
+#---------------------------------------------------------------------
+
+def server_html_page_for_session(session, resources, title, template=BASE_TEMPLATE,
+                                 template_variables=None):
+    render_item = RenderItem(
+        token = session.token,
+        roots = session.document.roots,
+        use_for_title = False,
+    )
+
+    if template_variables is None:
+        template_variables = {}
+
+    bundle = bundle_resources(resources)
+    return html_page_for_render_items(bundle, {}, [render_item], title,
+        template=template, template_variables=template_variables)
+
+def autoload_js_script(resources, token, element_id, app_path, absolute_url):
+    resources = Resources.from_bokeh(resources)
+    bundle = bundle_resources(resources)
+
+    render_items = [RenderItem(token=token, elementid=element_id, use_for_title=False)]
+    bundle.add(Script(script_for_render_items({}, render_items, app_path=app_path, absolute_url=absolute_url)))
+
+    return AUTOLOAD_JS.render(bundle=bundle, elementid=element_id)
+
+# Patch Application to handle session callbacks
+class Application(BkApplication):
+
+    async def on_session_created(self, session_context):
+        for cb in state._on_session_created:
+            cb(session_context)
+        await super().on_session_created(session_context)
+
+bokeh.command.util.Application = Application
+
+
+class SessionPrefixHandler:
+
+    @contextmanager
+    def _session_prefix(self):
+        prefix = self.request.uri.replace(self.application_context._url, '')
+        if not prefix.endswith('/'):
+            prefix += '/'
+        base_url = urljoin('/', prefix)
+        rel_path = '/'.join(['..'] * self.application_context._url.strip('/').count('/'))
+        old_url, old_rel = state.base_url, state.rel_path
+        with edit_readonly(state):
+            state.base_url = base_url
+            state.rel_path = rel_path
+        try:
+            yield
+        finally:
+            with edit_readonly(state):
+                state.base_url = old_url
+                state.rel_path = old_rel
+
+# Patch Bokeh DocHandler URL
+class DocHandler(BkDocHandler, SessionPrefixHandler):
+
+    @authenticated
+    async def get(self, *args, **kwargs):
+        with self._session_prefix():
+            session = await self.get_session()
+            resources = Resources.from_bokeh(self.application.resources())
+            page = server_html_page_for_session(
+                session, resources=resources, title=session.document.title,
+                template=session.document.template,
+                template_variables=session.document.template_variables
+            )
+        self.set_header("Content-Type", 'text/html')
+        self.write(page)
+
+per_app_patterns[0] = (r'/?', DocHandler)
+
+# Patch Bokeh Autoload handler
+class AutoloadJsHandler(BkAutoloadJsHandler, SessionPrefixHandler):
+    ''' Implements a custom Tornado handler for the autoload JS chunk
+
+    '''
+
+    async def get(self, *args, **kwargs):
+        element_id = self.get_argument("bokeh-autoload-element", default=None)
+        if not element_id:
+            self.send_error(status_code=400, reason='No bokeh-autoload-element query parameter')
+            return
+
+        app_path = self.get_argument("bokeh-app-path", default="/")
+        absolute_url = self.get_argument("bokeh-absolute-url", default=None)
+
+        if absolute_url:
+            server_url = '{uri.scheme}://{uri.netloc}'.format(uri=urlparse(absolute_url))
+        else:
+            server_url = None
+
+        with self._session_prefix():
+            session = await self.get_session()
+            resources = Resources.from_bokeh(self.application.resources(server_url))
+            js = autoload_js_script(resources, session.token, element_id, app_path, absolute_url)
+
+        self.set_header("Content-Type", 'application/javascript')
+        self.write(js)
+
+per_app_patterns[3] = (r'/autoload.js', AutoloadJsHandler)
+
+def modify_document(self, doc):
+    from bokeh.io.doc import set_curdoc as bk_set_curdoc
+    from ..config import config
+
+    if config.autoreload:
+        path = self._runner.path
+        argv = self._runner._argv
+        handler = type(self)(filename=path, argv=argv)
+        self._runner = handler._runner
+
+    module = self._runner.new_module()
+
+    # If no module was returned it means the code runner has some permanent
+    # unfixable problem, e.g. the configured source code has a syntax error
+    if module is None:
+        return
+
+    # One reason modules are stored is to prevent the module
+    # from being gc'd before the document is. A symptom of a
+    # gc'd module is that its globals become None. Additionally
+    # stored modules are used to provide correct paths to
+    # custom models resolver.
+    sys.modules[module.__name__] = module
+    doc._modules.append(module)
+
+    old_doc = curdoc()
+    bk_set_curdoc(doc)
+    old_io = self._monkeypatch_io()
+
+    if config.autoreload:
+        set_curdoc(doc)
+        state.onload(autoreload_watcher)
+
+    try:
+        def post_check():
+            newdoc = curdoc()
+            # script is supposed to edit the doc not replace it
+            if newdoc is not doc:
+                raise RuntimeError("%s at '%s' replaced the output document" % (self._origin, self._runner.path))
+
+        def handle_exception(handler, e):
+            from bokeh.application.handlers.handler import handle_exception
+            from ..pane import HTML
+
+            # Clean up
+            del sys.modules[module.__name__]
+            doc._modules.remove(module)
+            bokeh.application.handlers.code_runner.handle_exception = handle_exception
+            tb = html.escape(traceback.format_exc())
+
+            # Serve error
+            HTML(
+                f'<b>{type(e).__name__}</b>: {e}</br><pre style="overflow-y: scroll">{tb}</pre>',
+                css_classes=['alert', 'alert-danger'], sizing_mode='stretch_width'
+            ).servable()
+
+        if config.autoreload:
+            bokeh.application.handlers.code_runner.handle_exception = handle_exception
+        self._runner.run(module, post_check)
+    finally:
+        self._unmonkeypatch_io(old_io)
+        bk_set_curdoc(old_doc)
+
+CodeHandler.modify_document = modify_document
+
 #---------------------------------------------------------------------
 # Public API
 #---------------------------------------------------------------------
 
+def init_doc(doc):
+    doc = doc or curdoc()
+    if not doc.session_context:
+        return doc
+
+    session_id = doc.session_context.id
+    sessions = state.session_info['sessions']
+    if session_id not in sessions:
+        return doc
+
+    sessions[session_id].update({
+        'started': dt.datetime.now().timestamp()
+    })
+    doc.on_event('document_ready', state._init_session)
+    return doc
+
+@contextmanager
+def set_curdoc(doc):
+    state.curdoc = doc
+    yield
+    state.curdoc = None
+
+def with_lock(func):
+    """
+    Wrap a callback function to execute with a lock allowing the
+    function to modify bokeh models directly.
+
+    Arguments
+    ---------
+    func: callable
+      The callable to wrap
+
+    Returns
+    -------
+    wrapper: callable
+      Function wrapped to execute without a Document lock.
+    """
+    if inspect.iscoroutinefunction(func):
+        @wraps(func)
+        async def wrapper(*args, **kw):
+            return await func(*args, **kw)
+    else:
+        @wraps(func)
+        def wrapper(*args, **kw):
+            return func(*args, **kw)
+    wrapper.lock = True
+    return wrapper
+
 
 @contextmanager
 def unlocked():
@@ -180,7 +407,7 @@ def serve(panels, port=0, address=None, websocket_origin=None, loop=None,
     """
     Allows serving one or more panel objects on a single server.
     The panels argument should be either a Panel object or a function
-    returning a Panel object or a dictionary of these two. If a 
+    returning a Panel object or a dictionary of these two. If a
     dictionary is supplied the keys represent the slugs at which
     each app is served, e.g. `serve({'app': panel1, 'app2': panel2})`
     will serve apps at /app and /app2 on the server.
@@ -359,7 +586,7 @@ def get_server(panel, port=0, address=None, websocket_origin=None,
                     raise KeyError(
                         "Keys of the title dictionnary and of the apps "
                         f"dictionary must match. No {slug} key found in the "
-                        "title dictionnary.") 
+                        "title dictionary.")
             else:
                 title_ = title
             slug = slug if slug.startswith('/') else '/'+slug
@@ -374,9 +601,17 @@ def get_server(panel, port=0, address=None, websocket_origin=None,
                     extra_patterns.append(('^'+slug+'.*', ProxyFallbackHandler,
                                            dict(fallback=wsgi, proxy=slug)))
                     continue
-            apps[slug] = Application(FunctionHandler(partial(_eval_panel, app, server_id, title_, location)))
+            if isinstance(app, pathlib.Path):
+                app = str(app) # enables serving apps from Paths
+            if (isinstance(app, str) and (app.endswith(".py") or app.endswith(".ipynb"))
+                and os.path.isfile(app)):
+                apps[slug] = build_single_handler_application(app)
+            else:
+                handler = FunctionHandler(partial(_eval_panel, app, server_id, title_, location))
+                apps[slug] = Application(handler)
     else:
-        apps = {'/': Application(FunctionHandler(partial(_eval_panel, panel, server_id, title, location)))}
+        handler = FunctionHandler(partial(_eval_panel, panel, server_id, title, location))
+        apps = {'/': Application(handler)}
 
     extra_patterns += get_static_routes(static_dirs)
 
@@ -423,7 +658,8 @@ def get_server(panel, port=0, address=None, websocket_origin=None,
     server = Server(apps, port=port, **opts)
     if verbose:
         address = server.address or 'localhost'
-        print("Launching server at http://%s:%s" % (address, server.port))
+        url = f"http://{address}:{server.port}{server.prefix}"
+        print(f"Launching server at {url}")
 
     state._servers[server_id] = (server, panel, [])
 
@@ -456,7 +692,7 @@ class StoppableThread(threading.Thread):
     """Thread class with a stop() method."""
 
     def __init__(self, io_loop=None, **kwargs):
-        super(StoppableThread, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self.io_loop = io_loop
 
     def run(self):
