@@ -30,10 +30,9 @@ class GridBox(ListPanel):
 
     _bokeh_model = BkGridBox
 
-    _rename = {'objects': 'children'}
+    _rename = {'objects': 'children', 'nrows': None, 'ncols': None}
 
-    _source_transforms = {'scroll': None, 'objects': None,
-                         'nrows': None, 'ncols': None}
+    _source_transforms = {'scroll': None, 'objects': None}
 
     @classmethod
     def _flatten_grid(cls, layout, nrows=None, ncols=None):
@@ -132,10 +131,9 @@ class GridBox(ListPanel):
         if root is None:
             root = model
         objects = self._get_objects(model, [], doc, root, comm)
-        model.children = self._get_children(objects, self.nrows, self.ncols)
-        props = {k: v for k, v in self._init_properties().items()
-                 if k not in ('nrows', 'ncols')}
-        model.update(**self._process_param_change(props))
+        properties = self._process_param_change(self._init_params())
+        properties['children'] = self._get_children(objects, self.nrows, self.ncols)
+        model.update(**properties)
         self._models[root.ref['id']] = (model, parent)
         self._link_props(model, self._linked_props, doc, root, comm)
         return model
@@ -155,10 +153,17 @@ class GridBox(ListPanel):
 
         with hold(doc):
             msg = {k: v for k, v in msg.items() if k not in ('nrows', 'ncols')}
-            super(Panel, self)._update_model(events, msg, root, model, doc, comm)
-            ref = root.ref['id']
-            if ref in state._views:
-                state._views[ref][0]._preprocess(root)
+            update = Panel._batch_update
+            Panel._batch_update = True
+            try:
+                super(Panel, self)._update_model(events, msg, root, model, doc, comm)
+                if update:
+                    return
+                ref = root.ref['id']
+                if ref in state._views:
+                    state._views[ref][0]._preprocess(root)
+            finally:
+                Panel._batch_update = update
 
 
 class GridSpec(Panel):
@@ -188,7 +193,7 @@ class GridSpec(Panel):
     def __init__(self, **params):
         if 'objects' not in params:
             params['objects'] = OrderedDict()
-        super(GridSpec, self).__init__(**params)
+        super().__init__(**params)
         self._updating = False
         self._update_nrows()
         self._update_ncols()
@@ -215,14 +220,14 @@ class GridSpec(Panel):
             self.nrows = max(max_yidx) if max_yidx else (1 if len(self.objects) else 0)
         self._updating = False
 
-    def _init_properties(self):
-        properties = super(GridSpec, self)._init_properties()
+    def _init_params(self):
+        params = super()._init_params()
         if self.sizing_mode not in ['fixed', None]:
-            if 'min_width' not in properties and 'width' in properties:
-                properties['min_width'] = properties['width']
-            if 'min_height' not in properties and 'height' in properties:
-                properties['min_height'] = properties['height']
-        return properties
+            if 'min_width' not in params and 'width' in params:
+                params['min_width'] = params['width']
+            if 'min_height' not in params and 'height' in params:
+                params['min_height'] = params['height']
+        return params
 
     def _get_objects(self, model, old_objects, doc, root, comm=None):
         from ..pane.base import RerenderError
@@ -303,7 +308,7 @@ class GridSpec(Panel):
         return grid
 
     def _cleanup(self, root):
-        super(GridSpec, self)._cleanup(root)
+        super()._cleanup(root)
         for p in self.objects.values():
             p._cleanup(root)
 
@@ -432,9 +437,9 @@ class GridSpec(Panel):
             grid = clone.grid
             grid[t:b, l:r] += 1
 
-        overlap_grid = grid>1
+        overlap_grid = grid > 1
         new_objects = OrderedDict(self.objects)
-        if (overlap_grid).any():
+        if overlap_grid.any():
             overlapping = ''
             objects = []
             for (yidx, xidx) in zip(*np.where(overlap_grid)):
@@ -460,7 +465,10 @@ class GridSpec(Panel):
                 objects = [list(subgrid)[0][0]] if subgrid else []
             else:
                 objects = [list(o)[0][0] for o in subgrid.flatten()]
-            for dkey in objects:
-                del new_objects[dkey]
+            for dkey in set(objects):
+                try:
+                    del new_objects[dkey]
+                except KeyError:
+                    continue
         new_objects[key] = panel(obj)
         self.objects = new_objects
