@@ -2,20 +2,27 @@ export {AcePlot} from "./ace"
 export {Audio} from "./audio"
 export {Card} from "./card"
 export {CommManager} from "./comm_manager"
+export {DataTabulator} from "./tabulator"
+export {DatetimePicker} from "./datetime_picker"
 export {DeckGLPlot} from "./deckgl"
 export {ECharts} from "./echarts"
 export {HTML} from "./html"
+export {IDOM} from "./idom"
 export {IPyWidget} from "./ipywidget"
 export {JSON} from "./json"
 export {FileDownload} from "./file_download"
 export {KaTeX} from "./katex"
 export {Location} from "./location"
 export {MathJax} from "./mathjax"
+export {Perspective} from "./perspective"
 export {Player} from "./player"
 export {PlotlyPlot} from "./plotly"
 export {Progress} from "./progress"
 export {SingleSelect} from "./singleselect"
+export {SpeechToText} from "./speech_to_text"
 export {State} from "./state"
+export {TextToSpeech} from "./text_to_speech"
+export {TrendIndicator} from "./trend"
 export {VegaPlot} from "./vega"
 export {Video} from "./video"
 export {VideoStream} from "./videostream"
