@@ -2,8 +2,6 @@
 Pane class which render various markup languages including HTML,
 Markdown, and also regular strings.
 """
-from __future__ import absolute_import, division, unicode_literals
-
 import json
 import textwrap
 
@@ -46,7 +44,7 @@ class DivPaneBase(PaneBase):
         self._models[root.ref['id']] = (model, parent)
         return model
 
-    def _update(self, model):
+    def _update(self, ref=None, model=None):
         model.update(**self._get_properties())
 
 
@@ -74,7 +72,7 @@ class HTML(DivPaneBase):
             return False
 
     def _get_properties(self):
-        properties = super(HTML, self)._get_properties()
+        properties = super()._get_properties()
         text = '' if self.object is None else self.object
         if hasattr(text, '_repr_html_'):
             text = text._repr_html_()
@@ -161,7 +159,7 @@ class DataFrame(HTML):
     ]
 
     def __init__(self, object=None, **params):
-        super(DataFrame, self).__init__(object, **params)
+        super().__init__(object, **params)
         self._stream = None
         self._setup_stream()
 
@@ -189,12 +187,12 @@ class DataFrame(HTML):
         self._stream.sink(self._set_object)
 
     def _get_model(self, doc, root=None, parent=None, comm=None):
-        model = super(DataFrame, self)._get_model(doc, root, parent, comm)
+        model = super()._get_model(doc, root, parent, comm)
         self._setup_stream()
         return model
 
     def _cleanup(self, model):
-        super(DataFrame, self)._cleanup(model)
+        super()._cleanup(model)
         if not self._models and self._stream:
             self._stream.destroy()
             self._stream = None
@@ -241,7 +239,7 @@ class Str(DivPaneBase):
         return True
 
     def _get_properties(self):
-        properties = super(Str, self)._get_properties()
+        properties = super()._get_properties()
         if self.object is None:
             text = ''
         else:
@@ -269,7 +267,7 @@ class Markdown(DivPaneBase):
 
     _target_transforms = {'object': None}
 
-    _rerender_params = ['object', 'dedent', 'extensions']
+    _rerender_params = ['object', 'dedent', 'extensions', 'css_classes']
 
     @classmethod
     def applies(cls, obj):
@@ -289,7 +287,7 @@ class Markdown(DivPaneBase):
             data = data._repr_markdown_()
         if self.dedent:
             data = textwrap.dedent(data)
-        properties = super(Markdown, self)._get_properties()
+        properties = super()._get_properties()
         properties['style'] = properties.get('style', {})
         css_classes = properties.pop('css_classes', []) + ['markdown']
         html = markdown.markdown(data, extensions=self.extensions,
@@ -323,6 +321,8 @@ class JSON(DivPaneBase):
     _bokeh_model = _BkJSON
     _rename = {"name": None, "object": "text", "encoder": None}
 
+    _rerender_params = ['object', 'depth', 'encoder', 'hover_preview', 'theme']
+
     @classmethod
     def applies(cls, obj, **params):
         if isinstance(obj, (list, dict)):
@@ -338,11 +338,12 @@ class JSON(DivPaneBase):
             return None
 
     def _get_properties(self):
-        properties = super(JSON, self)._get_properties()
-        if isinstance(self.object, string_types):
-            text = self.object
-        else:
-            text = json.dumps(self.object or {}, cls=self.encoder)
-        depth = float('inf') if self.depth < 0 else self.depth
+        properties = super()._get_properties()
+        try:
+            data = json.loads(self.object)
+        except Exception:
+            data = self.object
+        text = json.dumps(data or {}, cls=self.encoder)
+        depth = None if self.depth < 0 else self.depth
         return dict(text=text, theme=self.theme, depth=depth,
                     hover_preview=self.hover_preview, **properties)
