@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, unicode_literals
-
 import sys
 
 import param
@@ -9,9 +7,10 @@ from bokeh.models import ColumnDataSource
 from pyviz_comms import JupyterComm
 
 from ..viewable import Layoutable
-from ..util import string_types
+from ..util import lazy_load, string_types
 from .base import PaneBase
 
+
 def ds_as_cds(dataset):
     """
     Converts Vega dataset into Bokeh ColumnDataSource data
@@ -137,24 +136,14 @@ class Vega(PaneBase):
             props['sizing_mode'] = 'stretch_height'
 
     def _get_model(self, doc, root=None, parent=None, comm=None):
-        if 'panel.models.vega' not in sys.modules:
-            if isinstance(comm, JupyterComm):
-                self.param.warning('VegaPlot was not imported on instantiation '
-                                   'and may not render in a notebook. Restart '
-                                   'the notebook kernel and ensure you load '
-                                   'it as part of the extension using:'
-                                   '\n\npn.extension(\'vega\')\n')
-            from ..models.vega import VegaPlot
-        else:
-            VegaPlot = getattr(sys.modules['panel.models.vega'], 'VegaPlot')
-
+        VegaPlot = lazy_load('panel.models.vega', 'VegaPlot', isinstance(comm, JupyterComm))
         sources = {}
         if self.object is None:
             json = None
         else:
             json = self._to_json(self.object)
             self._get_sources(json, sources)
-        props = self._process_param_change(self._init_properties())
+        props = self._process_param_change(self._init_params())
         self._get_dimensions(json, props)
         model = VegaPlot(data=json, data_sources=sources, **props)
         if root is None:
