@@ -2,8 +2,6 @@
 Defines the Param pane which converts Parameterized classes into a
 set of widgets.
 """
-from __future__ import absolute_import, division, unicode_literals
-
 import os
 import sys
 import json
@@ -12,6 +10,7 @@ import inspect
 import itertools
 
 from collections import OrderedDict, defaultdict, namedtuple
+from contextlib import contextmanager
 from six import string_types
 
 import param
@@ -19,18 +18,20 @@ import param
 from param.parameterized import classlist, discard_events
 
 from .io import init_doc, state
-from .layout import Row, Panel, Tabs, Column
+from .layout import Column, Panel, Row, Spacer, Tabs
 from .pane.base import PaneBase, ReplacementPane
 from .util import (
     abbreviated_repr, full_groupby, get_method_owner, is_parameterized,
     param_name, recursive_parameterized
 )
-from .viewable import Layoutable
+from .reactive import Reactive
+from .viewable import Layoutable, Viewable
 from .widgets import (
     Button, Checkbox, ColorPicker, DataFrame, DatePicker,
-    DatetimeInput, DateRangeSlider, FileSelector, FloatSlider,
-    IntInput, IntSlider, LiteralInput, MultiSelect, RangeSlider,
-    Select, FloatInput, StaticText, TextInput, Toggle, Widget
+    DatetimeInput, DateRangeSlider, DiscreteSlider, FileSelector,
+    FloatSlider, IntInput, IntSlider, LiteralInput, MultiSelect,
+    RangeSlider, Select, FloatInput, StaticText, TextInput, Toggle,
+    Widget
 )
 from .widgets.button import _ButtonBase
 
@@ -57,6 +58,31 @@ def LiteralInputTyped(pobj):
     return LiteralInput
 
 
+@contextmanager
+def set_values(*parameterizeds, **param_values):
+    """
+    Temporarily sets parameter values to the specified values on all
+    supplied Parameterized objects.
+
+    Arguments
+    ---------
+    parameterizeds: tuple(param.Parameterized)
+        Any number of parameterized objects.
+    param_values: dict
+        A dictionary of parameter names and temporary values.
+    """
+    old = []
+    for parameterized in parameterizeds:
+        old_values = {p: getattr(parameterized, p) for p in param_values}
+        old.append((parameterized, old_values))
+        parameterized.param.set_param(**param_values)
+    try:
+        yield
+    finally:
+        for parameterized, old_values in old:
+            parameterized.param.set_param(**old_values)
+
+
 class Param(PaneBase):
     """
     Param panes render a Parameterized class to a set of widgets which
@@ -165,7 +191,7 @@ class Param(PaneBase):
 
         if object and 'name' not in params:
             params['name'] = param_name(object.name)
-        super(Param, self).__init__(object, **params)
+        super().__init__(object, **params)
         self._updating = []
 
         # Construct Layout
@@ -214,8 +240,9 @@ class Param(PaneBase):
     # Callback API
     #----------------------------------------------------------------
 
+    @property
     def _synced_params(self):
-        ignored_params = ['default_layout']
+        ignored_params = ['default_layout', 'loading']
         return [p for p in Layoutable.param if p not in ignored_params]
 
     def _update_widgets(self, *events):
@@ -399,6 +426,7 @@ class Param(PaneBase):
         else:
             widget = widget_class(**kwargs)
         widget._param_pane = self
+        widget._param_name = p_name
 
         watchers = self._callbacks
 
@@ -427,6 +455,7 @@ class Param(PaneBase):
 
         def link(change, watchers=[watcher]):
             updates = {}
+            widget = self._widgets[p_name]
             if change.what == 'constant':
                 updates['disabled'] = change.new
             elif change.what == 'precedence':
@@ -434,22 +463,23 @@ class Param(PaneBase):
                     widget in self._widget_box.objects):
                     self._widget_box.pop(widget)
                 elif change.new >= self.display_threshold:
-                    precedence = lambda k: self.object.param['name' if k == '_title' else k].precedence
-                    params = self._ordered_params
-                    if self.show_name:
-                        params.insert(0, '_title')
-                    widgets = []
-                    for k in params:
-                        if precedence(k) is None or precedence(k) >= self.display_threshold:
-                            widgets.append(self._widgets[k])
-                    self._widget_box.objects = widgets
+                    self._rerender()
                 return
             elif change.what == 'objects':
                 updates['options'] = p_obj.get_range()
             elif change.what == 'bounds':
                 start, end = p_obj.get_soft_bounds()
-                updates['start'] = start
-                updates['end'] = end
+                supports_bounds = hasattr(widget, 'start')
+                if start is None or end is None:
+                    rerender = supports_bounds
+                else:
+                    rerender = not supports_bounds
+                if supports_bounds:
+                    updates['start'] = start
+                    updates['end'] = end
+                if rerender:
+                    self._rerender_widget(p_name)
+                    return
             elif change.what == 'step':
                 updates['step'] = p_obj.step
             elif change.what == 'label':
@@ -530,6 +560,27 @@ class Param(PaneBase):
     # Model API
     #----------------------------------------------------------------
 
+    def _rerender(self):
+        precedence = lambda k: self.object.param['name' if k == '_title' else k].precedence
+        params = self._ordered_params
+        if self.show_name:
+            params.insert(0, '_title')
+        widgets = []
+        for k in params:
+            if precedence(k) is None or precedence(k) >= self.display_threshold:
+                widgets.append(self._widgets[k])
+        self._widget_box.objects = widgets
+
+    def _rerender_widget(self, p_name):
+        watchers = []
+        for w in self._callbacks:
+            if w.inst is self._widgets[p_name]:
+                w.inst.param.unwatch(w)
+            else:
+                watchers.append(w)
+        self._widgets[p_name] = self.widget(p_name)
+        self._rerender()
+    
     def _get_widgets(self):
         """Return name,widget boxes for all parameters (i.e., a property sheet)"""
         # Format name specially
@@ -549,7 +600,7 @@ class Param(PaneBase):
 
     def _cleanup(self, root):
         self.layout._cleanup(root)
-        super(Param, self)._cleanup(root)
+        super()._cleanup(root)
 
     #----------------------------------------------------------------
     # Public API
@@ -622,12 +673,20 @@ class ParamMethod(ReplacementPane):
     return any object which itself can be rendered as a Pane.
     """
 
+    lazy = param.Boolean(default=False, doc="""
+        Whether to lazily evaluate the contents of the object
+        only when it is required for rendering.""")
+
+    loading_indicator = param.Boolean(default=False, doc="""
+        Whether to show loading indicator while pane is updating.""")
+
     def __init__(self, object=None, **params):
-        super(ParamMethod, self).__init__(object, **params)
+        super().__init__(object, **params)
+        self._evaled = not self.lazy
         self._link_object_params()
         if object is not None:
             self._validate_object()
-            self._update_inner(self.eval(object))
+            self._replace_pane(not self.lazy)
 
     @param.depends('object', watch=True)
     def _validate_object(self):
@@ -662,6 +721,19 @@ class ParamMethod(ReplacementPane):
                 kwargs = {n: getattr(dep.owner, dep.name) for n, dep in kw_deps.items()}
         return function(*args, **kwargs)
 
+    def _replace_pane(self, *args, force=False):
+        self._evaled = bool(self._models) or force or not self.lazy
+        if self._evaled:
+            self._inner_layout.loading = self.loading_indicator
+            try:
+                if self.object is None:
+                    new_object = Spacer()
+                else:
+                    new_object = self.eval(self.object)
+                self._update_inner(new_object)
+            finally:
+                self._inner_layout.loading = False
+
     def _update_pane(self, *events):
         callbacks = []
         for watcher in self._callbacks:
@@ -672,8 +744,7 @@ class ParamMethod(ReplacementPane):
             obj.param.unwatch(watcher)
         self._callbacks = callbacks
         self._link_object_params()
-        if object is not None:
-            self._update_inner(self.eval(self.object))
+        self._replace_pane()
 
     def _link_object_params(self):
         parameterized = get_method_owner(self.object)
@@ -704,16 +775,24 @@ class ParamMethod(ReplacementPane):
                     self._callbacks.append(watcher)
                     for p in params:
                         deps.append(p)
-            new_object = self.eval(self.object)
-            self._update_inner(new_object)
+            self._replace_pane()
 
         for _, params in full_groupby(params, lambda x: (x.inst or x.cls, x.what)):
             p = params[0]
             pobj = (p.inst or p.cls)
             ps = [_p.name for _p in params]
+            if isinstance(pobj, Reactive) and self.loading_indicator:
+                props = {p: 'loading' for p in ps if p in pobj._linkable_params}
+                if props:
+                    pobj.jslink(self._inner_layout, **props)
             watcher = pobj.param.watch(update_pane, ps, p.what)
             self._callbacks.append(watcher)
 
+    def _get_model(self, doc, root=None, parent=None, comm=None):
+        if not self._evaled:
+            self._replace_pane(force=True)
+        return super()._get_model(doc, root, parent, comm)
+
     #----------------------------------------------------------------
     # Public API
     #----------------------------------------------------------------
@@ -735,10 +814,6 @@ class ParamFunction(ParamMethod):
 
     priority = 0.6
 
-    def _replace_pane(self, *args):
-        new_object = self.eval(self.object)
-        self._update_inner(new_object)
-
     def _link_object_params(self):
         deps = self.object._dinfo
         dep_params = list(deps['dependencies']) + list(deps.get('kw', {}).values())
@@ -746,7 +821,13 @@ class ParamFunction(ParamMethod):
         for dep in dep_params:
             grouped[id(dep.owner)].append(dep)
         for group in grouped.values():
-            watcher = group[0].owner.param.watch(self._replace_pane, [dep.name for dep in group])
+            pobj = group[0].owner
+            watcher = pobj.param.watch(self._replace_pane, [dep.name for dep in group])
+            if isinstance(pobj, Reactive) and self.loading_indicator:
+                props = {dep.name: 'loading' for dep in group
+                         if dep.name in pobj._linkable_params}
+                if props:
+                    pobj.jslink(self._inner_layout, **props)
             self._callbacks.append(watcher)
 
     #----------------------------------------------------------------
@@ -817,3 +898,28 @@ class JSONInit(param.Parameterized):
                 parameterized.param.set_param(**{name:value})
             except ValueError as e:
                 warnobj.warning(str(e))
+
+
+def link_param_method(root_view, root_model):
+    """
+    This preprocessor jslinks ParamMethod loading parameters to any
+    widgets generated from those parameters ensuring that the loading
+    indicator is enabled client side.
+    """
+    methods = root_view.select(lambda p: isinstance(p, ParamMethod) and p.loading_indicator)
+    widgets = root_view.select(lambda w: isinstance(w, Widget) and getattr(w, '_param_pane', None) is not None)
+
+    for widget in widgets:
+        for method in methods:
+            for cb in method._callbacks:
+                pobj = cb.cls if cb.inst is None else cb.inst
+                if widget._param_pane.object is pobj and widget._param_name in cb.parameter_names:
+                    if isinstance(widget, DiscreteSlider):
+                        w = widget._slider
+                    else:
+                        w = widget
+                    if 'value' in w._linkable_params:
+                        w.jslink(method._inner_layout, value='loading')
+
+
+Viewable._preprocessing_hooks.insert(0, link_param_method)
