@@ -2,15 +2,12 @@
 Templates allow multiple Panel objects to be embedded into custom HTML
 documents.
 """
-from __future__ import absolute_import, division, unicode_literals
-
 import os
 import sys
 import uuid
 
 from collections import OrderedDict
 from functools import partial
-from urllib.parse import urljoin
 
 import param
 
@@ -21,10 +18,10 @@ from jinja2.environment import Template as _Template
 from six import string_types
 from pyviz_comms import JupyterCommManager as _JupyterCommManager
 
-from ..config import config, panel_extension
+from ..config import _base_config, config, panel_extension
 from ..io.model import add_to_doc
 from ..io.notebook import render_template
-from ..io.resources import CDN_DIST, LOCAL_DIST
+from ..io.resources import CDN_DIST, LOCAL_DIST, BUNDLE_DIR
 from ..io.save import save
 from ..io.state import state
 from ..layout import Column, ListLike, GridSpec
@@ -60,7 +57,7 @@ class BaseTemplate(param.Parameterized, ServableMixin):
     __abstract = True
 
     def __init__(self, template=None, items=None, nb_template=None, **params):
-        super(BaseTemplate, self).__init__(**params)
+        super().__init__(**params)
         if isinstance(template, string_types):
             self._code = template
             template = _Template(template)
@@ -150,6 +147,7 @@ class BaseTemplate(param.Parameterized, ServableMixin):
         preprocess_root = col.get_root(doc, comm)
         col._hooks.append(self._apply_hooks)
         ref = preprocess_root.ref['id']
+        objs = []
 
         for name, (obj, tags) in self._render_items.items():
             if self._apply_hooks not in obj._hooks:
@@ -165,17 +163,18 @@ class BaseTemplate(param.Parameterized, ServableMixin):
                 sub._models[ref] = submodel
                 if isinstance(sub, HoloViews) and mref in sub._plots:
                     sub._plots[ref] = sub._plots.get(mref)
-            col.objects.append(obj)
             obj._documents[doc] = model
             model.name = name
             model.tags = tags
             self._apply_root(name, model, tags)
             add_to_doc(model, doc, hold=bool(comm))
+            objs.append(obj)
 
         # Here we ensure that the preprocessor is run across all roots
         # and set up session cleanup hooks for the fake root.
         state._fake_roots.append(ref)
         state._views[ref] = (col, preprocess_root, doc, comm)
+        col.objects = objs
         col._preprocess(preprocess_root)
         col._documents[doc] = preprocess_root
         doc.on_session_destroyed(col._server_destroy)
@@ -323,6 +322,11 @@ class BasicTemplate(BaseTemplate):
     feel without having to write any Jinja2 template themselves.
     """
 
+    config = param.ClassSelector(default=_base_config(), class_=_base_config,
+                                 constant=True, doc="""
+        Configuration object declaring custom CSS and JS files to load
+        specifically for this template.""")
+
     busy_indicator = param.ClassSelector(default=LoadingSpinner(width=20, height=20),
                                          class_=BooleanIndicator, constant=True, doc="""
         Visual indicator of application busy state.""")
@@ -398,13 +402,27 @@ class BasicTemplate(BaseTemplate):
 
     location = param.Boolean(default=True, readonly=True)
 
+    #############
+    # Resources #
+    #############
+
+    # Resource locations for bundled resources
+    _CDN = CDN_DIST
+    _LOCAL = LOCAL_DIST
+
+    # pathlib.Path pointing to local CSS file(s)
     _css = None
 
+    # pathlib.Path pointing to local JS file(s)
+    _js = None
+
+    # pathlib.Path pointing to local Jinja2 template
     _template = None
 
-    _modifiers = {}
+    # External resources
+    _resources = {'css': {}, 'js': {}, 'js_modules': {}, 'tarball': {}}
 
-    _resources = {'css': {}, 'js': {}}
+    _modifiers = {}
 
     __abstract = True
 
@@ -426,12 +444,11 @@ class BasicTemplate(BaseTemplate):
             params['modal'] = ListLike()
         else:
             params['modal'] = self._get_params(params['modal'], self.param.modal.class_)
-        super(BasicTemplate, self).__init__(template=template, **params)
+        super().__init__(template=template, **params)
         if self.busy_indicator:
             state.sync_busy(self.busy_indicator)
         self._js_area = HTML(margin=0, width=0, height=0)
         self._render_items['js_area'] = (self._js_area, [])
-        self._update_vars()
         self._update_busy()
         self.main.param.watch(self._update_render_items, ['objects'])
         self.modal.param.watch(self._update_render_items, ['objects'])
@@ -441,46 +458,123 @@ class BasicTemplate(BaseTemplate):
         self.sidebar.param.trigger('objects')
         self.header.param.trigger('objects')
         self.modal.param.trigger('objects')
-        self.param.watch(self._update_vars, ['title', 'site', 'header_background',
-                                             'header_color', 'main_max_width'])
 
     def _init_doc(self, doc=None, comm=None, title=None, notebook=False, location=True):
         title = title or self.title
-        doc = super(BasicTemplate, self)._init_doc(doc, comm, title, notebook, location)
+        self._update_vars()
+        doc = super()._init_doc(doc, comm, title, notebook, location)
         if self.theme:
-            theme = self.theme.find_theme(type(self))
+            theme = self._get_theme()
             if theme and theme.bokeh_theme:
                 doc.theme = theme.bokeh_theme
         return doc
 
+    def _get_theme(self):
+        return self.theme.find_theme(type(self))()
+
     def _template_resources(self):
         name = type(self).__name__.lower()
         resources = _settings.resources(default="server")
-        base_url = state.base_url[1:] if state.base_url.startswith('/') else state.base_url        
         if resources == 'server':
-            dist_path = urljoin(base_url, LOCAL_DIST)
+            if state.rel_path:
+                dist_path = f'{state.rel_path}/{self._LOCAL}'
+            else:
+                dist_path = self._LOCAL
         else:
-            dist_path = CDN_DIST
+            dist_path = self._CDN
 
         # External resources
-        css_files = dict(self._resources['css'])
+        css_files = dict(self._resources.get('css', {}))
         for cssname, css in css_files.items():
             css_path = url_path(css)
-            css_files[cssname] = dist_path + f'bundled/{name}/{css_path}'
-        js_files = dict(self._resources['js'])
+            if (BUNDLE_DIR / 'css' / css_path.replace('/', os.path.sep)).is_file():
+                css_files[cssname] = dist_path + f'bundled/css/{css_path}'
+        js_files = dict(self._resources.get('js', {}))
         for jsname, js in js_files.items():
             js_path = url_path(js)
-            js_files[jsname] = dist_path + f'bundled/{name}/{js_path}'
+            if (BUNDLE_DIR / 'js' / js_path.replace('/', os.path.sep)).is_file():
+                js_files[jsname] = dist_path + f'bundled/js/{js_path}'
+        js_modules = dict(self._resources.get('js_modules', {}))
+        for jsname, js in js_modules.items():
+            js_path = url_path(js)
+            if jsname in self._resources.get('tarball', {}):
+                js_path += '/index.mjs'
+            else:
+                js_path += '.mjs'
+            if os.path.isfile(BUNDLE_DIR / js_path.replace('/', os.path.sep)):
+                js_modules[jsname] = dist_path + f'bundled/js/{js_path}'
+        for name, js in self.config.js_files.items():
+            if not '//' in js and state.rel_path:
+                js = f'{state.rel_path}/{js}'
+            js_files[name] = js
+        for name, js in self.config.js_modules.items():
+            if not '//' in js and state.rel_path:
+                js = f'{state.rel_path}/{js}'
+            js_modules[name] = js
+        extra_css = []
+        for css in list(self.config.css_files):
+            if not '//' in css and state.rel_path:
+                css = f'{state.rel_path}/{css}'
+            extra_css.append(css)
+        raw_css = list(self.config.raw_css)
 
         # CSS files
-        base_css = os.path.basename(self._css)
-        css_files['base'] = dist_path + f'bundled/{name}/{base_css}'
+        base_css = self._css
+        if not isinstance(base_css, list):
+            base_css = [base_css] if base_css else []
+        for css in base_css:
+            tmpl_name = name
+            for cls in type(self).__mro__[1:-5]:
+                tmpl_css = cls._css if isinstance(cls._css, list) else [cls._css]
+                if css in tmpl_css:
+                    tmpl_name = cls.__name__.lower()
+            css_file = os.path.basename(css)
+            if (BUNDLE_DIR / tmpl_name / css_file).is_file():
+                css_files[f'base_{css_file}'] = dist_path + f'bundled/{tmpl_name}/{css_file}'
+            else:
+                with open(css, encoding='utf-8') as f:
+                    raw_css.append(f.read())
+
+        # JS files
+        base_js = self._js
+        if not isinstance(base_js, list):
+            base_js = [base_js] if base_js else []
+        for js in base_js:
+            tmpl_name = name
+            for cls in type(self).__mro__[1:-5]:
+                tmpl_js = cls._js if isinstance(cls._js, list) else [cls._js]
+                if js in tmpl_js:
+                    tmpl_name = cls.__name__.lower()
+            js = os.path.basename(js)
+            if (BUNDLE_DIR / tmpl_name / js).is_file():
+                js_files[f'base_{js}'] = dist_path + f'bundled/{tmpl_name}/{js}'
+
         if self.theme:
             theme = self.theme.find_theme(type(self))
-            if theme and theme.css:
-                basename = os.path.basename(theme.css)
-                css_files['theme'] = dist_path + f'bundled/{name}/{basename}'
-        return {'css': css_files, 'js': js_files}
+            if theme:
+                if theme.base_css:
+                    basename = os.path.basename(theme.base_css)
+                    owner = theme.param.base_css.owner.__name__.lower()
+                    if (BUNDLE_DIR / owner / basename).is_file():
+                        css_files['theme_base'] = dist_path + f'bundled/{owner}/{basename}'
+                    else:
+                        with open(theme.base_css, encoding='utf-8') as f:
+                            raw_css.append(f.read())
+                if theme.css:
+                    basename = os.path.basename(theme.css)
+                    if (BUNDLE_DIR / name / basename).is_file():
+                        css_files['theme'] = dist_path + f'bundled/{name}/{basename}'
+                    else:
+                        with open(theme.base_css, encoding='utf-8') as f:
+                            raw_css.append(f.read())
+
+        return {
+            'css': css_files,
+            'extra_css': extra_css,
+            'raw_css': raw_css,
+            'js': js_files,
+            'js_modules': js_modules
+        }
 
     def _update_vars(self, *args):
         self._render_variables['app_title'] = self.title
@@ -660,7 +754,7 @@ class Template(BaseTemplate):
     """
 
     def __init__(self, template=None, nb_template=None, items=None, **params):
-        super(Template, self).__init__(template=template, nb_template=nb_template, items=items, **params)
+        super().__init__(template=template, nb_template=nb_template, items=items, **params)
         items = {} if items is None else items
         for name, item in items.items():
             self.add_panel(name, item)
