@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, unicode_literals
-
 import base64
 import json
 
@@ -207,6 +205,22 @@ def test_json_pane(document, comm):
     assert model.text == '{"b": 3}'
     assert pane._models[model.ref['id']][0] is model
 
+    pane.object = {"test": "can't show this"}
+    assert model.text == '{"test": "can\'t show this"}'
+    assert pane._models[model.ref['id']][0] is model
+
+    pane.object = ["can't show this"]
+    assert model.text == '["can\'t show this"]'
+    assert pane._models[model.ref['id']][0] is model
+
+    pane.object = "can't show this"
+    assert model.text == '"can\'t show this"'
+    assert pane._models[model.ref['id']][0] is model
+
+    pane.object = "can show this"
+    assert model.text == '"can show this"'
+    assert pane._models[model.ref['id']][0] is model
+
     # Cleanup
     pane._cleanup(model)
     assert pane._models == {}
