@@ -5,8 +5,8 @@ import param
 from bokeh.models import Div
 from panel.layout import Tabs, WidgetBox
 from panel.reactive import Reactive
-from panel.viewable import Layoutable
-from panel.widgets import Checkbox, StaticText, TextInput
+from panel.viewable import Viewable
+from panel.widgets import Checkbox, StaticText, TextInput, IntInput
 
 
 def test_link():
@@ -92,29 +92,35 @@ def test_text_input_controls():
     assert len(controls) == 2
     wb1, wb2 = controls
     assert isinstance(wb1, WidgetBox)
-    assert len(wb1) == 4
-    name, disabled, text1, text2 = wb1
-    if text1.name == 'Placeholder':
-        placeholder, value = text1, text2
-    else:
-        placeholder, value = text2, text1
+    assert len(wb1) == 7
+    name, disabled, visible, *(ws) = wb1
 
     assert isinstance(name, StaticText)
     assert isinstance(disabled, Checkbox)
-    assert isinstance(value, TextInput)
-    assert isinstance(placeholder, TextInput)
+    assert isinstance(visible, Checkbox)
 
-    text_input.disabled = True
-    assert disabled.value
+    not_checked = []
+    for w in ws:
+        print(w.name)
+        if w.name == 'Value':
+            assert isinstance(w, TextInput)
+            text_input.value = "New value"
+            assert w.value == "New value"
+        elif w.name == 'Value input':
+            assert isinstance(w, TextInput)
+        elif w.name == 'Placeholder':
+            assert isinstance(w, TextInput)
+            text_input.placeholder = "Test placeholder..."
+            assert w.value == "Test placeholder..."
+        elif w.name == 'Max length':
+            assert isinstance(w, IntInput)
+        else:
+            not_checked.append(w)
 
-    text_input.placeholder = "Test placeholder..."
-    assert placeholder.value == "Test placeholder..."
-
-    text_input.value = "New value"
-    assert value.value == "New value"
+    assert not not_checked
 
     assert isinstance(wb2, WidgetBox)
-    assert len(wb2) == len(list(Layoutable.param)) + 1
+    assert len(wb2) == len(list(Viewable.param)) + 1
 
 
 
