@@ -1,12 +1,11 @@
-from __future__ import absolute_import, division, unicode_literals
-
 import pytest
 from datetime import datetime, date
 
 from bokeh.models.widgets import FileInput as BkFileInput
 from panel.widgets import (
-    Checkbox, DatePicker, DatetimeInput, DatetimeRangeInput, FileInput,
-    LiteralInput, TextInput, StaticText
+    Checkbox, DatePicker, DatetimeInput, DatetimePicker,
+    DatetimeRangeInput, DatetimeRangePicker, FileInput,
+    LiteralInput, TextInput, StaticText,
 )
 
 
