@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, unicode_literals
-
 from io import BytesIO, StringIO
 from base64 import b64encode
 
@@ -43,7 +41,7 @@ def test_audio_url(document, comm):
 def test_progress_bounds():
     progress = Progress()
     progress.max = 200
-    assert progress.param.value.bounds == (0, 200)
+    assert progress.param.value.bounds == (-1, 200)
     progress.value = 120
     assert progress.value == 120
 
@@ -163,4 +161,4 @@ def test_file_download_data():
 
     file_download.data = None
     file_download._clicks += 1
-    assert file_download.data is not None
\ No newline at end of file
+    assert file_download.data is not None
