@@ -2,13 +2,11 @@
 The input widgets generally allow entering arbitrary information into
 a text field or similar.
 """
-from __future__ import absolute_import, division, unicode_literals
-
 import ast
 import json
 
 from base64 import b64decode
-from datetime import datetime
+from datetime import datetime, date
 from six import string_types
 
 import param
@@ -17,37 +15,38 @@ from bokeh.models.widgets import (
     CheckboxGroup as _BkCheckboxGroup, ColorPicker as _BkColorPicker,
     DatePicker as _BkDatePicker, Div as _BkDiv, TextInput as _BkTextInput,
     PasswordInput as _BkPasswordInput, Spinner as _BkSpinner,
-    FileInput as _BkFileInput, TextAreaInput as _BkTextAreaInput)
+    FileInput as _BkFileInput, TextAreaInput as _BkTextAreaInput,
+    NumericInput as _BkNumericInput)
 
-from ..util import as_unicode
-from .base import Widget
+from ..layout import Column
+from ..util import param_reprs, as_unicode
+from .base import Widget, CompositeWidget
+from ..models import DatetimePicker as _bkDatetimePicker
 
 
 class TextInput(Widget):
 
-    value = param.String(default='', allow_None=True)
+    max_length = param.Integer(default=5000, doc="""
+      Max count of characters in the input field.""")
 
-    placeholder = param.String(default='')
+    placeholder = param.String(default='', doc="""
+      Placeholder for empty input field.""")
+
+    value = param.String(default='', allow_None=True, doc="""
+      Initial or entered text value updated when <enter> key is pressed.""")
+
+    value_input = param.String(default='', allow_None=True, doc="""
+      Initial or entered text value updated on every key press.""")
 
     _widget_type = _BkTextInput
 
 
-class PasswordInput(Widget):
-
-    value = param.String(default='', allow_None=True)
-
-    placeholder = param.String(default='')
+class PasswordInput(TextInput):
 
     _widget_type = _BkPasswordInput
 
 
-class TextAreaInput(Widget):
-
-    value = param.String(default='', allow_None=True)
-
-    placeholder = param.String(default='')
-
-    max_length = param.Integer(default=5000)
+class TextAreaInput(TextInput):
 
     _widget_type = _BkTextAreaInput
 
@@ -56,9 +55,13 @@ class FileInput(Widget):
 
     accept = param.String(default=None)
 
-    filename = param.String(default=None)
+    filename = param.ClassSelector(default=None, class_=(str, list),
+                               is_instance=True)
 
-    mime_type = param.String(default=None)
+    mime_type = param.ClassSelector(default=None, class_=(str, list),
+                               is_instance=True)
+
+    multiple = param.Boolean(default=False)
 
     value = param.Parameter(default=None)
 
@@ -69,7 +72,7 @@ class FileInput(Widget):
     _rename = {'name': None, 'filename': None}
 
     def _process_param_change(self, msg):
-        msg = super(FileInput, self)._process_param_change(msg)
+        msg = super()._process_param_change(msg)
         if 'value' in msg:
             msg.pop('value')
         if 'mime_type' in msg:
@@ -77,13 +80,16 @@ class FileInput(Widget):
         return msg
 
     def _filter_properties(self, properties):
-        properties = super(FileInput, self)._filter_properties(properties)
+        properties = super()._filter_properties(properties)
         return properties + ['value', 'mime_type', 'filename']
 
     def _process_property_change(self, msg):
-        msg = super(FileInput, self)._process_property_change(msg)
+        msg = super()._process_property_change(msg)
         if 'value' in msg:
-            msg['value'] = b64decode(msg['value'])
+            if isinstance(msg['value'], string_types):
+                msg['value'] = b64decode(msg['value'])
+            else:
+                msg['value'] = [b64decode(content) for content in msg['value']]
         return msg
 
     def save(self, filename):
@@ -119,7 +125,7 @@ class StaticText(Widget):
     _widget_type = _BkDiv
 
     def _process_param_change(self, msg):
-        msg = super(StaticText, self)._process_property_change(msg)
+        msg = super()._process_property_change(msg)
         if 'value' in msg:
             text = as_unicode(msg.pop('value'))
             partial = self._format.replace('{value}', '').format(title=self.name)
@@ -137,14 +143,18 @@ class DatePicker(Widget):
 
     end = param.CalendarDate(default=None)
 
-    _source_transforms = {'value': None, 'start': None, 'end': None}
+    disabled_dates = param.List(default=None, class_=(date, str))
+
+    enabled_dates = param.List(default=None, class_=(date, str))
+
+    _source_transforms = {}
 
     _rename = {'start': 'min_date', 'end': 'max_date', 'name': 'title'}
 
     _widget_type = _BkDatePicker
 
     def _process_property_change(self, msg):
-        msg = super(DatePicker, self)._process_property_change(msg)
+        msg = super()._process_property_change(msg)
         if 'value' in msg:
             if isinstance(msg['value'], string_types):
                 msg['value'] = datetime.date(datetime.strptime(msg['value'], '%Y-%m-%d'))
@@ -161,30 +171,118 @@ class ColorPicker(Widget):
     _rename = {'value': 'color', 'name': 'title'}
 
 
-class Spinner(Widget):
+class _NumericInputBase(Widget):
 
-    start = param.Number(default=None, doc="""
-        Optional minimum allowable value.""")
-
-    end = param.Number(default=None, doc="""
-        Optional maximum allowable value.""")
-
-    value = param.Number(default=0, doc="""
+    value = param.Number(default=0, allow_None=True, doc="""
         The initial value of the spinner.""")
 
-    step = param.Number(default=1, doc="""
-        The step added or subtracted to the current value.""")
+    placeholder = param.String(default='0', doc="""
+        Placeholder for empty input field.""")
+
+    format = param.String(default=None, allow_None=True, doc="""
+        Number formating : http://numbrojs.com/old-format.html .""")
+
+    _rename = {'name': 'title', 'start': 'low', 'end': 'high'}
+
+    _widget_type = _BkNumericInput
+
+    __abstract = True
+
+
+class _IntInputBase(_NumericInputBase):
+
+    value = param.Integer(default=0, allow_None=True, doc="""
+        The initial value of the spinner.""")
+
+    start = param.Integer(default=None, allow_None=True, doc="""
+        Optional minimum allowable value.""")
+
+    end = param.Integer(default=None, allow_None=True, doc="""
+        Optional maximum allowable value.""")
+
+    mode = param.String(default='int', constant=True, doc="""
+        Define the type of number which can be enter in the input""")
+
+    __abstract = True
+
+
+class _FloatInputBase(_NumericInputBase):
+
+    value = param.Number(default=0, allow_None=True, doc="""
+        The initial value of the spinner.""")
+
+    start = param.Number(default=None, allow_None=True, doc="""
+        Optional minimum allowable value.""")
+
+    end = param.Number(default=None, allow_None=True, doc="""
+        Optional maximum allowable value.""")
+
+    mode = param.String(default='float', constant=True, doc="""
+        Define the type of number which can be enter in the input""")
+
+    __abstract = True
+
+
+class _SpinnerBase(_NumericInputBase):
+
+    page_step_multiplier = param.Integer(default=10, bounds=(0, None), doc="""
+        Defines the multiplication factor applied to step when the page up
+        and page down keys are pressed.""")
+
+    wheel_wait = param.Integer(default=100, doc="""
+        Defines the debounce time in ms before updating `value_throttled` when
+        the mouse wheel is used to change the input.""")
 
     _widget_type = _BkSpinner
 
-    _rename = {'name': 'title', 'start': 'low', 'end': 'high'}
+    __abstract = True
 
     def __init__(self, **params):
         if params.get('value') is None:
             value = params.get('start', self.value)
             if value is not None:
                 params['value'] = value
-        super(Spinner, self).__init__(**params)
+        if 'value' in params and 'value_throttled' in self.param:
+            params['value_throttled'] = params['value']
+        super().__init__(**params)
+
+    def __repr__(self, depth=0):
+        return '{cls}({params})'.format(cls=type(self).__name__,
+                                        params=', '.join(param_reprs(self, ['value_throttled'])))
+
+    def _update_model(self, events, msg, root, model, doc, comm):
+        if 'value_throttled' in msg:
+            del msg['value_throttled']
+
+        return super()._update_model(events, msg, root, model, doc, comm)
+
+
+class IntInput(_SpinnerBase, _IntInputBase):
+
+    step = param.Integer(default=1)
+
+    value_throttled = param.Integer(default=None, constant=True)
+
+
+class FloatInput(_SpinnerBase, _FloatInputBase):
+
+    step = param.Number(default=0.1)
+
+    value_throttled = param.Number(default=None, constant=True)
+
+
+class NumberInput(_SpinnerBase):
+
+    def __new__(self, **params):
+        param_list = ["value", "start", "stop", "step"]
+        if all(isinstance(params.get(p, 0), int) for p in param_list):
+            return IntInput(**params)
+        else:
+            return FloatInput(**params)
+
+
+# Backward compatibility
+Spinner = NumberInput
 
 
 class LiteralInput(Widget):
@@ -212,7 +310,7 @@ class LiteralInput(Widget):
     _widget_type = _BkTextInput
 
     def __init__(self, **params):
-        super(LiteralInput, self).__init__(**params)
+        super().__init__(**params)
         self._state = ''
         self._validate(None)
         self._callbacks.append(self.param.watch(self._validate, 'value'))
@@ -229,7 +327,7 @@ class LiteralInput(Widget):
                              (types, new, type(new).__name__))
 
     def _process_property_change(self, msg):
-        msg = super(LiteralInput, self)._process_property_change(msg)
+        msg = super()._process_property_change(msg)
         new_state = ''
         if 'value' in msg:
             value = msg.pop('value')
@@ -264,15 +362,15 @@ class LiteralInput(Widget):
         return msg
 
     def _process_param_change(self, msg):
-        msg = super(LiteralInput, self)._process_param_change(msg)
+        msg = super()._process_param_change(msg)
         if 'value' in msg:
-            value = '' if msg['value'] is None else msg['value']
+            value = msg['value']
             if isinstance(value, string_types):
                 value = repr(value)
             elif self.serializer == 'json':
-                value = json.dumps(value, sort_keys=True)
+                value = json.dumps(value)
             else:
-                value = as_unicode(value)
+                value = '' if value is None else as_unicode(value)
             msg['value'] = value
         msg['title'] = self.name
         return msg
@@ -301,7 +399,7 @@ class DatetimeInput(LiteralInput):
                'start': None, 'end': None, 'serializer': None}
 
     def __init__(self, **params):
-        super(DatetimeInput, self).__init__(**params)
+        super().__init__(**params)
         self.param.watch(self._validate, 'value')
         self._validate(None)
 
@@ -351,6 +449,67 @@ class DatetimeInput(LiteralInput):
         return msg
 
 
+class DatetimeRangeInput(CompositeWidget):
+
+    value = param.Tuple(default=(None, None), length=2)
+
+    start = param.Date(default=None)
+
+    end = param.Date(default=None)
+
+    format = param.String(default='%Y-%m-%d %H:%M:%S', doc="""
+        Datetime format used for parsing and formatting the datetime.""")
+
+    _composite_type = Column
+
+    def __init__(self, **params):
+        self._text = StaticText(margin=(5, 0, 0, 0), style={'white-space': 'nowrap'})
+        self._start = DatetimeInput(sizing_mode='stretch_width', margin=(5, 0, 0, 0))
+        self._end = DatetimeInput(sizing_mode='stretch_width', margin=(5, 0, 0, 0))
+        if 'value' not in params:
+            params['value'] = (params['start'], params['end'])
+        super().__init__(**params)
+        self._msg = ''
+        self._composite.extend([self._text, self._start, self._end])
+        self._updating = False
+        self._update_widgets()
+        self._update_label()
+
+    @param.depends('name', '_start.name', '_end.name', watch=True)
+    def _update_label(self):
+        self._text.value = f'{self.name}{self._start.name}{self._end.name}{self._msg}'
+
+    @param.depends('_start.value', '_end.value', watch=True)
+    def _update(self):
+        if self._updating:
+            return
+        if (self._start.value is not None and
+            self._end.value is not None and
+            self._start.value > self._end.value):
+            self._msg = ' (start of range must be <= end)'
+            self._update_label()
+            return
+        elif self._msg:
+            self._msg = ''
+            self._update_label()
+        try:
+            self._updating = True
+            self.value = (self._start.value, self._end.value)
+        finally:
+            self._updating = False
+
+    @param.depends('value', 'start', 'end', 'name', 'format', watch=True)
+    def _update_widgets(self):
+        if self._updating:
+            return
+        try:
+            self._updating = True
+            self._start.param.set_param(value=self.value[0], start=self.start, end=self.end, format=self.format)
+            self._end.param.set_param(value=self.value[1], start=self.start, end=self.end, format=self.format)
+        finally:
+            self._updating = False
+
+
 class Checkbox(Widget):
 
     value = param.Boolean(default=False)
@@ -366,7 +525,7 @@ class Checkbox(Widget):
     _widget_type = _BkCheckboxGroup
 
     def _process_property_change(self, msg):
-        msg = super(Checkbox, self)._process_property_change(msg)
+        msg = super()._process_property_change(msg)
         if 'value' in msg:
             msg['value'] = 0 in msg.pop('value')
         if 'name' in msg:
@@ -374,14 +533,14 @@ class Checkbox(Widget):
         return msg
 
     def _process_param_change(self, msg):
-        msg = super(Checkbox, self)._process_param_change(msg)
+        msg = super()._process_param_change(msg)
         if 'active' in msg:
             msg['active'] = [0] if msg.pop('active', None) else []
         if 'labels' in msg:
             msg['labels'] = [msg.pop('labels')]
         return msg
 
-    def _get_embed_state(self, root, max_opts=3):
+    def _get_embed_state(self, root, values=None, max_opts=3):
         return (self, self._models[root.ref['id']][0], [False, True],
                 lambda x: str(0 in x.active).lower(), 'active',
                 "String(cb_obj.active.indexOf(0) >= 0)")
