@@ -2,8 +2,6 @@
 Defines various Select widgets which allow choosing one or more items
 from a list of options.
 """
-from __future__ import absolute_import, division, unicode_literals
-
 import re
 
 from collections import OrderedDict
@@ -14,10 +12,12 @@ from bokeh.models.widgets import (
     AutocompleteInput as _BkAutocompleteInput, CheckboxGroup as _BkCheckboxGroup,
     CheckboxButtonGroup as _BkCheckboxButtonGroup, MultiSelect as _BkMultiSelect,
     RadioButtonGroup as _BkRadioButtonGroup, RadioGroup as _BkRadioBoxGroup,
-    Select as _BkSelect, MultiChoice as _BkMultiChoice)
+    Select as _BkSelect, MultiChoice as _BkMultiChoice
+)
 
 from ..layout import Column, VSpacer
-from ..util import as_unicode, isIn, indexOf
+from ..models import SingleSelect as _BkSingleSelect
+from ..util import as_unicode, isIn, indexOf, bokeh_version
 from .base import Widget, CompositeWidget
 from .button import _ButtonBase, Button
 from .input import TextInput, TextAreaInput
@@ -45,22 +45,23 @@ class SelectBase(Widget):
         return OrderedDict(zip(self.labels, self.values))
 
 
-class Select(SelectBase):
+
+class SingleSelectBase(SelectBase):
 
     value = param.Parameter(default=None)
 
     _supports_embed = True
 
-    _widget_type = _BkSelect
+    __abstract = True
 
     def __init__(self, **params):
-        super(Select, self).__init__(**params)
+        super().__init__(**params)
         values = self.values
         if self.value is None and None not in values and values:
             self.value = values[0]
 
     def _process_param_change(self, msg):
-        msg = super(Select, self)._process_param_change(msg)
+        msg = super()._process_param_change(msg)
         labels, values = self.labels, self.values
         unique = len(set(self.unicode_values)) == len(labels)
         if 'value' in msg:
@@ -70,8 +71,9 @@ class Select(SelectBase):
                 msg['value'] = unicode_values[indexOf(val, values)]
             elif values:
                 self.value = self.values[0]
-            elif self.value is not None:
+            else:
                 self.value = None
+                msg['value'] = ''
 
         if 'options' in msg:
             if isinstance(self.options, dict):
@@ -86,7 +88,7 @@ class Select(SelectBase):
             if values:
                 if not isIn(val, values):
                     self.value = values[0]
-            elif val is not None:
+            else:
                 self.value = None
         return msg
 
@@ -95,12 +97,12 @@ class Select(SelectBase):
         return [as_unicode(v) for v in self.values]
 
     def _process_property_change(self, msg):
-        msg = super(Select, self)._process_property_change(msg)
+        msg = super()._process_property_change(msg)
         if 'value' in msg:
             if not self.values:
                 pass
-            elif msg['value'] is None:
-                msg['value'] = self.values[0]
+            elif msg['value'] == '':
+                msg['value'] = self.values[0] if self.values else None
             else:
                 if isIn(msg['value'], self.unicode_values):
                     idx = indexOf(msg['value'], self.unicode_values)
@@ -110,19 +112,48 @@ class Select(SelectBase):
         msg.pop('options', None)
         return msg
 
-    def _get_embed_state(self, root, max_opts=3):
-        return (self, self._models[root.ref['id']][0], self.values,
+    def _get_embed_state(self, root, values=None, max_opts=3):
+        if values is None:
+            values = self.values
+        elif any(v not in self.values for v in values):
+            raise ValueError("Supplied embed states were not found "
+                             "in the %s widgets values list." %
+                             type(self).__name__)
+        return (self, self._models[root.ref['id']][0], values,
                 lambda x: x.value, 'value', 'cb_obj.value')
 
 
-class _MultiSelectBase(Select):
+class Select(SingleSelectBase):
+
+    size = param.Integer(default=1, bounds=(1, None), doc="""
+        Declares how many options are displayed at the same time.
+        If set to 1 displays options as dropdown otherwise displays
+        scrollable area.""")
+
+    @property
+    def _widget_type(self):
+        return _BkSelect if self.size == 1 else _BkSingleSelect
+
+    def __init__(self, **params):
+        super().__init__(**params)
+        if self.size == 1:
+            self.param.size.constant = True
+
+    def _process_param_change(self, msg):
+        msg = super()._process_param_change(msg)
+        if msg.get('size') == 1:
+            msg.pop('size')
+        return msg
+
+
+class _MultiSelectBase(SingleSelectBase):
 
     value = param.List(default=[])
 
     _supports_embed = False
 
     def _process_param_change(self, msg):
-        msg = super(Select, self)._process_param_change(msg)
+        msg = super(SingleSelectBase, self)._process_param_change(msg)
         labels, values = self.labels, self.values
         if 'value' in msg:
             msg['value'] = [labels[indexOf(v, values)] for v in msg['value']
@@ -135,7 +166,7 @@ class _MultiSelectBase(Select):
         return msg
 
     def _process_property_change(self, msg):
-        msg = super(Select, self)._process_property_change(msg)
+        msg = super(SingleSelectBase, self)._process_property_change(msg)
         if 'value' in msg:
             labels = self.labels
             msg['value'] = [self._items[v] for v in msg['value']
@@ -173,24 +204,43 @@ class MultiChoice(_MultiSelectBase):
     _widget_type = _BkMultiChoice
 
 
+_AutocompleteInput_rename = {'name': 'title', 'options': 'completions'}
+if bokeh_version < '2.3.0':
+    # disable restrict keyword
+    _AutocompleteInput_rename['restrict'] = None
+
 class AutocompleteInput(Widget):
 
+    case_sensitive = param.Boolean(default=True, doc="""
+        Enable or disable case sensitivity.""")
+
     min_characters = param.Integer(default=2, doc="""
         The number of characters a user must type before
         completions are presented.""")
 
-    options = param.List(default=[])
+    options = param.List(default=[], doc="""
+        A list of completion strings. This will be used to guide the
+        user upon typing the beginning of a desired value.""")
 
-    placeholder = param.String(default='')
+    placeholder = param.String(default='', doc="""
+        Placeholder for empty input field.""")
 
-    value = param.Parameter(default=None)
+    restrict = param.Boolean(default=True, doc="""
+        Set to False in order to allow users to enter text that is not
+        present in the list of completion strings.""")
+
+    value = param.String(default='', allow_None=True, doc="""
+      Initial or entered text value updated when <enter> key is pressed.""")
+
+    value_input = param.String(default='', allow_None=True, doc="""
+      Initial or entered text value updated on every key press.""")
 
     _widget_type = _BkAutocompleteInput
 
-    _rename = {'name': 'title', 'options': 'completions'}
+    _rename = _AutocompleteInput_rename
 
 
-class _RadioGroupBase(Select):
+class _RadioGroupBase(SingleSelectBase):
 
     _supports_embed = False
 
@@ -203,7 +253,7 @@ class _RadioGroupBase(Select):
     __abstract = True
 
     def _process_param_change(self, msg):
-        msg = super(Select, self)._process_param_change(msg)
+        msg = super(SingleSelectBase, self)._process_param_change(msg)
         values = self.values
         if 'active' in msg:
             value = msg['active']
@@ -215,14 +265,14 @@ class _RadioGroupBase(Select):
                 msg['active'] = None
 
         if 'labels' in msg:
-            msg['labels'] = list(msg['labels'])
+            msg['labels'] = self.labels
             value = self.value
             if not isIn(value, values):
                 self.value = None
         return msg
 
     def _process_property_change(self, msg):
-        msg = super(Select, self)._process_property_change(msg)
+        msg = super(SingleSelectBase, self)._process_property_change(msg)
         if 'value' in msg:
             index = msg['value']
             if index is None:
@@ -231,8 +281,14 @@ class _RadioGroupBase(Select):
                 msg['value'] = list(self.values)[index]
         return msg
 
-    def _get_embed_state(self, root, max_opts=3):
-        return (self, self._models[root.ref['id']][0], self.values,
+    def _get_embed_state(self, root, values=None, max_opts=3):
+        if values is None:
+            values = self.values
+        elif any(v not in self.values for v in values):
+            raise ValueError("Supplied embed states were not found in "
+                             "the %s widgets values list." %
+                             type(self).__name__)
+        return (self, self._models[root.ref['id']][0], values,
                 lambda x: x.active, 'active', 'cb_obj.active')
 
 
@@ -257,7 +313,7 @@ class RadioBoxGroup(_RadioGroupBase):
 
 
 
-class _CheckGroupBase(Select):
+class _CheckGroupBase(SingleSelectBase):
 
     value = param.List(default=[])
 
@@ -272,20 +328,20 @@ class _CheckGroupBase(Select):
     __abstract = True
 
     def _process_param_change(self, msg):
-        msg = super(Select, self)._process_param_change(msg)
+        msg = super(SingleSelectBase, self)._process_param_change(msg)
         values = self.values
         if 'active' in msg:
             msg['active'] = [indexOf(v, values) for v in msg['active']
                              if isIn(v, values)]
         if 'labels' in msg:
-            msg['labels'] = list(msg['labels'])
+            msg['labels'] = self.labels
             if any(not isIn(v, values) for v in self.value):
                 self.value = [v for v in self.value if isIn(v, values)]
         msg.pop('title', None)
         return msg
 
     def _process_property_change(self, msg):
-        msg = super(Select, self)._process_property_change(msg)
+        msg = super(SingleSelectBase, self)._process_property_change(msg)
         if 'value' in msg:
             values = self.values
             msg['value'] = [values[a] for a in msg['value']]
@@ -308,7 +364,7 @@ class CheckBoxGroup(_CheckGroupBase):
 
 
 
-class ToggleGroup(Select):
+class ToggleGroup(SingleSelectBase):
     """This class is a factory of ToggleGroup widgets.
 
     A ToggleGroup is a group of widgets which can be switched 'on' or 'off'.
@@ -384,7 +440,7 @@ class CrossSelector(CompositeWidget, MultiSelect):
        selected list.""")
 
     def __init__(self, **params):
-        super(CrossSelector, self).__init__(**params)
+        super().__init__(**params)
         # Compute selected and unselected values
 
         labels, values = self.labels, self.values
