@@ -3,8 +3,6 @@ Defines the Widget base class which provides bi-directional
 communication between the rendered dashboard and the Widget
 parameters.
 """
-from __future__ import absolute_import, division, unicode_literals
-
 from six import string_types
 
 import param
@@ -18,14 +16,13 @@ from bokeh.models.widgets import (
 
 from ..config import config
 from ..io import state
-from ..util import unicode_repr, value_as_datetime, value_as_date
+from ..util import param_reprs, unicode_repr, value_as_datetime, value_as_date
 from ..viewable import Layoutable
 from .base import Widget, CompositeWidget
 from ..layout import Column
 from .input import StaticText
 
 
-
 class _SliderBase(Widget):
 
     bar_color = param.Color(default="#e6e6e6", doc="""
@@ -54,7 +51,17 @@ class _SliderBase(Widget):
     def __init__(self, **params):
         if 'value' in params and 'value_throttled' in self.param:
             params['value_throttled'] = params['value']
-        super(_SliderBase, self).__init__(**params)
+        super().__init__(**params)
+
+    def __repr__(self, depth=0):
+        return '{cls}({params})'.format(cls=type(self).__name__,
+                                        params=', '.join(param_reprs(self, ['value_throttled'])))
+
+    def _update_model(self, events, msg, root, model, doc, comm):
+        if 'value_throttled' in msg:
+            del msg['value_throttled']
+
+        return super()._update_model(events, msg, root, model, doc, comm)
 
 
 class ContinuousSlider(_SliderBase):
@@ -69,7 +76,7 @@ class ContinuousSlider(_SliderBase):
     def __init__(self, **params):
         if 'value' not in params:
             params['value'] = params.get('start', self.start)
-        super(ContinuousSlider, self).__init__(**params)
+        super().__init__(**params)
 
     def _get_embed_state(self, root, values=None, max_opts=3):
         ref = root.ref['id']
@@ -118,6 +125,8 @@ class FloatSlider(ContinuousSlider):
 
     step = param.Number(default=0.1)
 
+    _rename = {'name': 'title'}
+
 
 class IntSlider(ContinuousSlider):
 
@@ -131,6 +140,8 @@ class IntSlider(ContinuousSlider):
 
     step = param.Integer(default=1)
 
+    _rename = {'name': 'title'}
+
     def _process_property_change(self, msg):
         msg = super(_SliderBase, self)._process_property_change(msg)
         if 'value' in msg:
@@ -151,6 +162,8 @@ class DateSlider(_SliderBase):
 
     end = param.Date(default=None)
 
+    _rename = {'name': 'title'}
+
     _source_transforms = {'value': None, 'value_throttled': None, 'start': None, 'end': None}
 
     _widget_type = _BkDateSlider
@@ -158,7 +171,7 @@ class DateSlider(_SliderBase):
     def __init__(self, **params):
         if 'value' not in params:
             params['value'] = params.get('start', self.start)
-        super(DateSlider, self).__init__(**params)
+        super().__init__(**params)
 
     def _process_property_change(self, msg):
         msg = super(_SliderBase, self)._process_property_change(msg)
@@ -194,7 +207,7 @@ class DiscreteSlider(CompositeWidget, _SliderBase):
 
     def __init__(self, **params):
         self._syncing = False
-        super(DiscreteSlider, self).__init__(**params)
+        super().__init__(**params)
         if 'formatter' not in params and all(isinstance(v, (int, np.int_)) for v in self.values):
             self.formatter = '%d'
         if self.value is None and None not in self.values and self.options:
@@ -345,20 +358,22 @@ class RangeSlider(_SliderBase):
 
     step = param.Number(default=0.1)
 
+    _rename = {'name': 'title'}
+
     _widget_type = _BkRangeSlider
 
     def __init__(self, **params):
         if 'value' not in params:
             params['value'] = (params.get('start', self.start),
                                params.get('end', self.end))
-        super(RangeSlider, self).__init__(**params)
+        super().__init__(**params)
         values = [self.value[0], self.value[1], self.start, self.end]
         if (all(v is None or isinstance(v, int) for v in values) and
             'step' not in params):
             self.step = 1
 
     def _process_property_change(self, msg):
-        msg = super(RangeSlider, self)._process_property_change(msg)
+        msg = super()._process_property_change(msg)
         if 'value' in msg:
             msg['value'] = tuple(msg['value'])
         if 'value_throttled' in msg:
@@ -400,16 +415,26 @@ class DateRangeSlider(_SliderBase):
     _source_transforms = {'value': None, 'value_throttled': None,
                          'start': None, 'end': None, 'step': None}
 
+    _rename = {'name': 'title'}
+
     _widget_type = _BkDateRangeSlider
 
     def __init__(self, **params):
         if 'value' not in params:
             params['value'] = (params.get('start', self.start),
                                params.get('end', self.end))
-        super(DateRangeSlider, self).__init__(**params)
+        super().__init__(**params)
+
+    def _process_param_change(self, msg):
+        msg = super()._process_param_change(msg)
+        if msg.get('value') == (None, None):
+            del msg['value']
+        if msg.get('value_throttled') == (None, None):
+            del msg['value_throttled']
+        return msg
 
     def _process_property_change(self, msg):
-        msg = super(DateRangeSlider, self)._process_property_change(msg)
+        msg = super()._process_property_change(msg)
         if 'value' in msg:
             v1, v2 = msg['value']
             msg['value'] = (value_as_datetime(v1), value_as_datetime(v2))
